﻿namespace ChartRepaint
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Drawing.Drawing2D;
    using System.Threading;
    using System.Windows.Forms;

    using Steema.TeeChart;
    using Steema.TeeChart.Styles;

    public partial class Form1 : Form
    {
        public readonly object _locker = new object();

        public List<FastLine> series = new List<FastLine>();

        public Form1()
        {
            InitializeComponent();

            this.tChart1.Aspect.SmoothingMode = SmoothingMode.HighSpeed;
            this.tChart1.Aspect.View3D = false;

            this.tChart1.Panel.MarginUnits = PanelMarginUnits.Pixels;
            this.tChart1.Panel.MarginLeft = 10;
            this.tChart1.Panel.MarginRight = 10;
            this.tChart1.Panel.MarginTop = 10;
            this.tChart1.Panel.MarginBottom = 10;

            this.tChart1.Axes.Bottom.Labels.DateTimeFormat = "H:m:s,f";

            this.series.Add((FastLine)this.tChart1[0]);
            this.series.Add((FastLine)this.tChart1[1]);
            this.series.Add((FastLine)this.tChart1[2]);
            this.series.Add((FastLine)this.tChart1[3]);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            Thread test = new Thread(DoWork);
            test.Start();
        }

        private void DoWork()
        {
            Random rnd1 = new Random();
            double f1 = rnd1.Next(100000);
            double f2 = rnd1.Next(100000);
            double f3 = rnd1.Next(100000);
            double f4 = rnd1.Next(100000);
            
            this.tChart1.Invoke(
                (MethodInvoker)delegate
                    {
                        foreach (var series in this.tChart1.Series)
                        {
                            ((FastLine)series).Clear();
                        }

                        this.tChart1.AutoRepaint = false;
                    });
            
            lock (_locker)
            {
                for (int i = 0; i < 300000; i++)
                {
                    double id = i;
                    this.series[0].Add(i, Math.Sin(id / f1));
                    this.series[1].Add(i, Math.Cos(id / f2));
                    this.series[2].Add(i, Math.Cos(id / f3));
                    this.series[3].Add(i, Math.Sin(id / f4));

                    if (i % 5000 == 0)
                    {
                        Debug.Print("Count : " + i);
                        Thread.Sleep(400);
                        Application.DoEvents();
                    }
                }
            }
            
            this.tChart1.Invoke(
                    (MethodInvoker)delegate
                        {
                            this.tChart1.AutoRepaint = true;
                            this.tChart1.Refresh();
                        });
        }

        private void tChart1_BeforeDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {
            Monitor.Enter(_locker);
            Debug.Print("Before Draw");
        }

        private void tChart1_AfterDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {
            Monitor.Exit(_locker);
            Debug.Print("After Draw");
        }
    }
}